@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileCopy.h,v 1.85 2010/05/20 13:53:15 lav Exp $ */
-
 /*
   classes defined here:
    FileCopy
@@ -169,7 +167,6 @@ protected:
       } state;
 
 private:
-   int max_buf;
    bool cont;
 
    xstring_c error_text;
@@ -291,7 +288,7 @@ class FileCopyPeerFA : public FileCopyPeer
    int Put_LL(const char *buf,int size);
    int PutEOF_LL();
 
-   FileAccess::fileinfo info;
+   FileSet info;
 
    bool fxp;   // FXP (ftp<=>ftp copy) active
 
@@ -415,4 +412,21 @@ public:
    void UseColor(bool c=true) { if(dl) dl->UseColor(c); }
 };
 
+class FileCopyPeerMemory : public FileCopyPeer
+{
+private:
+   int max_size;
+
+public:
+   FileCopyPeerMemory(int m) : FileCopyPeer(PUT), max_size(m) {}
+   FileCopyPeerMemory(const xstring& s) : FileCopyPeer(GET), max_size(0) {
+      Put(s);
+      PutEOF();
+      size=s.length();
+      pos=0;
+   }
+   int Do();
+   bool Done() { return true; }
+};
+
 #endif
