@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: NetAccess.h,v 1.51 2009/07/17 13:03:01 lav Exp $ */
-
 #ifndef NETACCESS_H
 #define NETACCESS_H
 
@@ -114,8 +111,6 @@ public:
 class GenericParseListInfo : public ListInfo
 {
 protected:
-   xarray<FA::fileinfo> get_info;
-
    int mode;
    SMTaskRef<IOBuffer> ubuf;
 
