@@ -1,11 +1,11 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2003-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: SFtp.h,v 1.28 2008/05/16 08:45:01 lav Exp $ */
-
 #ifndef SFTP_H
 #define SFTP_H
 
@@ -55,8 +52,8 @@ enum packet_type {
    SSH_FXP_RMDIR    =15,
    SSH_FXP_REALPATH =16,
    SSH_FXP_STAT     =17,
-   SSH_FXP_RENAME   =18,
-   SSH_FXP_READLINK =19,
+   SSH_FXP_RENAME   =18,   // v>=2
+   SSH_FXP_READLINK =19,   // v>=3
    SSH_FXP_SYMLINK  =20,   // v<=5
    SSH_FXP_LINK     =21,   // v>=6
    SSH_FXP_BLOCK    =22,   // v>=6
@@ -426,8 +423,8 @@ public:
       off_t    size;		    // present only if flag SIZE
       xstring  owner;		    // present only if flag OWNERGROUP // v>=4
       xstring  group;		    // present only if flag OWNERGROUP // v>=4
-      uid_t    uid;		    // present only if flag UIDGID // v<=3
-      gid_t    gid;		    // present only if flag UIDGID // v<=3
+      unsigned uid;		    // present only if flag UIDGID // v<=3
+      unsigned gid;		    // present only if flag UIDGID // v<=3
       unsigned permissions;	    // present only if flag PERMISSIONS
       time_t   atime;		    // present only if flag ACCESSTIME (ACMODTIME)
       unsigned atime_nseconds;	    // present only if flag SUBSECOND_TIMES
@@ -646,6 +643,21 @@ private:
 	    protocol_version=pv;
 	    flags=f;
 	 }
+      void ComputeLength();
+      void Pack(Buffer *b);
+   };
+   class Request_READLINK : public PacketSTRING
+   {
+   public:
+      Request_READLINK(const char *name) : PacketSTRING(SSH_FXP_READLINK,name) {}
+   };
+   class Request_SYMLINK : public Packet
+   {
+      xstring oldpath;
+      xstring newpath;
+   public:
+      Request_SYMLINK(const char *o,const char *n)
+      : Packet(SSH_FXP_SYMLINK), oldpath(o), newpath(n) {}
       void ComputeLength()
 	 {
 	    Packet::ComputeLength();
@@ -653,6 +665,21 @@ private:
 	 }
       void Pack(Buffer *b);
    };
+   class Request_LINK : public Packet
+   {
+      xstring oldpath;
+      xstring newpath;
+      bool symbolic;
+   public:
+      Request_LINK(const char *o,const char *n,bool s)
+      : Packet(SSH_FXP_LINK), oldpath(o), newpath(n), symbolic(s) {}
+      void ComputeLength()
+	 {
+	    Packet::ComputeLength();
+	    length+=4+strlen(oldpath)+4+strlen(newpath)+1;
+	 }
+      void Pack(Buffer *b);
+   };
 
    struct Expect;
    friend struct SFtp::Expect; // grant access to Packet.
@@ -667,6 +694,7 @@ private:
 	 HANDLE_STALE,
 	 DATA,
 	 INFO,
+	 INFO_READLINK,
 	 DEFAULT,
 	 WRITE_STATUS,
 	 IGNORE
