@@ -1,11 +1,11 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: history.cc,v 1.21 2008/05/22 13:20:40 lav Exp $ */
-
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -39,7 +36,7 @@ History::History()
    stamp=0;
    fd=-1;
    modified=false;
-   const char *home=get_lftp_home();
+   const char *home=get_lftp_data_dir();
    if(home)
       file.vset(home,"/cwd_history",NULL);
 }
@@ -138,16 +135,16 @@ void History::Set(const FileAccess *s,const FileAccess::Path &cwd)
 {
    if(cwd.path==0 || !strcmp(cwd.path,"~") || s->GetHostName()==0)
       return;
-   char *res=(char*)alloca(32+strlen(cwd.path)*3+xstrlen(cwd.url)+2);
-   sprintf(res,"%lu:",(unsigned long)time(0));
+   xstring res;
+   res.setf("%lu:",(unsigned long)time(0));
    if(!cwd.url)
    {
-      url::encode_string(cwd,res+strlen(res),URL_PATH_UNSAFE);
+      res.append_url_encoded(cwd,URL_PATH_UNSAFE);
       if(!cwd.is_file && url::dir_needs_trailing_slash(s->GetProto()))
-	 strcat(res,"/");
+	 res.append('/');
    }
    else
-      strcat(res,cwd.url);
+      res.append(cwd.url);
    super::Add(s->GetConnectURL(s->NO_PATH|s->NO_PASSWORD),res);
    modified=true;
 }
