@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,27 +14,24 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: lftp.cc,v 1.83 2009/07/21 08:41:12 lav Exp $ */
-
 #include <config.h>
 
 #include "modconfig.h"
 
 #include "trio.h"
+#include <sys/types.h>
+#include <sys/stat.h> // for mkdir()
 #include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h> // for mkdir()
-#include "xstring.h"
 #include <fcntl.h>
 #include <locale.h>
 #include <ctype.h>
 
+#include "xstring.h"
 #include "xmalloc.h"
 #include "alias.h"
 #include "CmdExec.h"
@@ -50,16 +47,19 @@
 #include "ConnectionSlot.h"
 #include "misc.h"
 #include "ArgV.h"
+#include "attach.h"
 
 #include "configmake.h"
 
 #include "lftp_rl.h"
 #include "complete.h"
 
+CDECL_BEGIN
+#include <glob.h>
+CDECL_END
 
 #define top_exec CmdExec::top
 
-
 void  hook_signals()
 {
    SignalHook::DoCount(SIGHUP);
@@ -71,8 +71,10 @@ ResDecl res_save_cwd_history
    ("cmd:save-cwd-history","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
 ResDecl res_save_rl_history
    ("cmd:save-rl-history","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
+ResDecl res_stifle_rl_history
+   ("cmd:stifle-rl-history","500",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
-class ReadlineFeeder : public CmdFeeder
+class ReadlineFeeder : public CmdFeeder, private ResClient
 {
    bool tty:1;
    bool ctty:1;
@@ -94,6 +96,7 @@ class ReadlineFeeder : public CmdFeeder
 	 lftp_add_history_nodups(for_history);
 	 for_history.set(0);
       }
+      Reconfig(0);
    }
 
 public:
@@ -116,6 +119,10 @@ public:
 	    lftp_rl_write_history();
       }
    }
+   bool IsInteractive() const
+   {
+      return tty;
+   }
    bool RealEOF()
    {
       return !tty || eof_count>3;
@@ -214,6 +221,9 @@ public:
 	    return;
 	 lftp_rl_clear();
       }
+   void Reconfig(const char *) {
+      lftp_rl_history_stifle(res_stifle_rl_history.Query(0));
+   }
 };
 bool ReadlineFeeder::readline_inited;
 
@@ -297,16 +307,67 @@ CMD(history)
 
    return 0;
 }
+CMD(attach)
+{
+   const char *pid_s=args->getarg(1);
+   if(!pid_s) {
+      xstring& path=AcceptTermFD::get_sock_path(1);
+      path.rtrim('1');
+      path.append('*');
+      glob_t g;
+      glob(path, 0, NULL, &g);
+      for(size_t i=0; i<g.gl_pathc; i++) {
+	 const char *sock_path=g.gl_pathv[i];
+	 pid_s=strrchr(sock_path,'-');
+	 if(!pid_s)
+	    continue;
+	 pid_s++;
+	 int p=atoi(pid_s);
+	 if(p<=1) {
+	    pid_s=0;
+	    continue;
+	 }
+	 if(kill(p,0)==-1) {
+	    if(errno==ESRCH) {
+	       eprintf("%s: removing stale socket `%s'.\n",args->a0(),sock_path);
+	       if(unlink(sock_path)==-1)
+		  eprintf("%s: unlink(%s): %s\n",args->a0(),sock_path,strerror(errno));
+	    }
+	    pid_s=0;
+	    continue;
+	 }
+	 pid_s=alloca_strdup(pid_s);
+	 break;
+      }
+      globfree(&g);
+      if(!pid_s) {
+	 eprintf("%s: no backgrounded lftp processes found.\n",args->a0());
+	 return 0;
+      }
+   }
+   int pid=atoi(pid_s);
+   SMTaskRef<SendTermFD> term_sender(new SendTermFD(pid));
+   while(!term_sender->Done()) {
+      SMTask::Schedule();
+      SMTask::Block();
+   }
+   exit_code=0;
+   if(term_sender->Failed()) {
+      eprintf("%s\n",term_sender->ErrorText());
+      exit_code=1;
+   }
+   return 0;
+}
 #undef args
 #undef exit_code
 #undef output
 #undef session
 #undef eprintf
 
+
 static void sig_term(int sig)
 {
-   time_t t=time(0);
-   printf(_("[%lu] Terminated by signal %d. %s"),(unsigned long)getpid(),sig,ctime(&t));
+   printf(_("[%u] Terminated by signal %d. %s\n"),(unsigned)getpid(),sig,SMTask::now.IsoDateTime());
    if(top_exec) {
       top_exec->KillAll();
       alarm(30);
@@ -318,7 +379,45 @@ static void sig_term(int sig)
    exit(1);
 }
 
-static void move_to_background()
+static void detach()
+{
+   SignalHook::Ignore(SIGINT);
+   SignalHook::Ignore(SIGHUP);
+   SignalHook::Ignore(SIGTSTP);
+
+   const char *home=get_lftp_data_dir();
+   if(home)
+   {
+      xstring& log=xstring::get_tmp(home);
+      if(access(log,F_OK)==-1)
+	 log.append("_log");
+      else
+	 log.append("/log");
+
+      int fd=open(log,O_WRONLY|O_APPEND|O_CREAT,0600);
+      if(fd>=0)
+      {
+	 dup2(fd,1); // stdout
+	 dup2(fd,2); // stderr
+	 if(fd!=1 && fd!=2)
+	    close(fd);
+      }
+      Log::global->ShowPID();
+      Log::global->ShowTime();
+      Log::global->ShowContext();
+   }
+   close(0);	  // close stdin.
+   open("/dev/null",O_RDONLY); // reopen it, just in case.
+
+#ifdef HAVE_SETSID
+   setsid();	  // start a new session.
+#endif
+
+   SignalHook::Handle(SIGTERM,sig_term);
+}
+
+SMTaskRef<AcceptTermFD> term_acceptor;
+static int move_to_background()
 {
    // notify jobs
    Job::lftpMovesToBackground_ToAll();
@@ -326,73 +425,55 @@ static void move_to_background()
    SMTask::RollAll(TimeInterval(1,0));
    // if all jobs terminated, don't really move to bg.
    if(Job::NumberOfJobs()==0)
-      return;
+      return 0;
+
+   top_exec->AtBackground();
+   top_exec->WaitDone();
+   if(Job::NumberOfJobs()==0)
+      return 0;
 
    fflush(stdout);
    fflush(stderr);
 
-   pid_t pid=fork();
+   static bool detached;
+   pid_t pid=0;
+   if(!detached)
+      pid=fork();
    switch(pid)
    {
    case(0): // child
    {
-      SignalHook::Ignore(SIGINT);
-      SignalHook::Ignore(SIGQUIT);
-      SignalHook::Ignore(SIGHUP);
-      SignalHook::Ignore(SIGTSTP);
-
-      const char *home=get_lftp_home();
-      if(home)
-      {
-	 char *log=(char*)alloca(strlen(home)+1+3+1);
-	 sprintf(log,"%s",home);
-	 if(access(log,F_OK)==-1)
-	    strcat(log,"_log");
-	 else
-	    strcat(log,"/log");
-
-	 int fd=open(log,O_WRONLY|O_APPEND|O_CREAT,0600);
-	 if(fd>=0)
-	 {
-	    dup2(fd,1); // stdout
-	    dup2(fd,2); // stderr
-	    if(fd!=1 && fd!=2)
-	       close(fd);
-	 }
-	 Log::global->ShowPID();
-	 Log::global->ShowTime();
-	 Log::global->ShowContext();
+      if(!detached) {
+	 detach();
+	 detached=true;
+	 printf(_("[%u] Started.  %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
       }
-      close(0);	  // close stdin.
-      open("/dev/null",O_RDONLY); // reopen it, just in case.
-
-#ifdef HAVE_SETSID
-      setsid();	  // start a new session.
-#endif
-
-      pid=getpid();
-      time_t t=time(0);
-      SignalHook::Handle(SIGTERM,sig_term);
-      printf(_("[%lu] Started.  %s"),(unsigned long)pid,ctime(&t));
+      if(!term_acceptor)
+	 term_acceptor=new AcceptTermFD();
       for(;;)
       {
 	 SMTask::Schedule();
 	 if(Job::NumberOfJobs()==0)
 	    break;
 	 SMTask::Block();
+	 if(term_acceptor->Accepted())
+	    return 1;
       }
-      t=time(0);
-      printf(_("[%lu] Finished. %s"),(unsigned long)pid,ctime(&t));
-      return;
+      top_exec->AtExitBg();
+      top_exec->AtTerminate();
+      top_exec->WaitDone();
+      printf(_("[%u] Finished. %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
+      return 0;
    }
    default: // parent
-      printf(_("[%lu] Moving to background to complete transfers...\n"),
-	       (unsigned long)pid);
+      printf(_("[%u] Moving to background to complete transfers...\n"),
+	       (unsigned)pid);
       fflush(stdout);
       _exit(0);
    case(-1):
       perror("fork()");
    }
+   return 0;
 }
 
 int lftp_slot(int count,int key)
@@ -446,6 +527,8 @@ int   main(int argc,char **argv)
 	 " -l  List the history (default).\n"
 	 "Optional argument cnt specifies the number of history lines to list,\n"
 	 "or \"all\" to list all entries.\n"));
+   CmdExec::RegisterCommand("attach",cmd_attach,"attach [PID]",
+      N_("Attach the terminal to specified backgrounded lftp process.\n"));
 
    top_exec=new CmdExec(0,0);
    hook_signals();
@@ -456,7 +539,7 @@ int   main(int argc,char **argv)
    const char *home=getenv("HOME");
    if(home)
       source_if_exist(dir_file(home,".lftprc"));
-   home=get_lftp_home();
+   home=get_lftp_config_dir();
    if(home)
       source_if_exist(dir_file(home,"rc"));
    top_exec->WaitDone();
@@ -470,6 +553,7 @@ int   main(int argc,char **argv)
    lftp_feeder=new ReadlineFeeder(args);
 
    top_exec->ExecParsed(args.borrow());
+revived:
    top_exec->WaitDone();
    int exit_code=top_exec->ExitCode();
 
@@ -479,11 +563,36 @@ int   main(int argc,char **argv)
    if(Job::NumberOfJobs()>0)
    {
       top_exec->SetInteractive(false);
-      move_to_background();
+      if(term_acceptor) {
+	 printf(_("[%u] Detaching from the terminal to complete transfers...\n"),
+	       (unsigned)getpid());
+	 fflush(stdout);
+	 term_acceptor->Detach();
+	 detach();
+	 printf(_("[%u] Detached from terminal. %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
+      }
+      if(move_to_background())
+      {
+	 top_exec->SetInteractive();
+	 top_exec->SetStatusLine(new StatusLine(1));
+	 top_exec->SetCmdFeeder(new ReadlineFeeder(args));
+	 goto revived;
+      }
+   }
+   else
+   {
+      top_exec->AtExitFg();
+      top_exec->AtTerminate();
+      top_exec->WaitDone();
    }
    top_exec->KillAll();
    top_exec=0;
 
+   if(term_acceptor && term_acceptor->Accepted()) {
+      printf(_("[%u] Exiting and detaching from the terminal.\n"),(unsigned)getpid());
+      fflush(stdout);
+   }
+
    Job::Cleanup();
    ConnectionSlot::Cleanup();
    SessionPool::ClearAll();
@@ -493,6 +602,5 @@ int   main(int argc,char **argv)
    SignalHook::Cleanup();
    Log::Cleanup();
    SMTask::Cleanup();
-
    return exit_code;
 }
