@@ -1,11 +1,11 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: lftp_tinfo.cc,v 1.5 2005/12/02 09:07:29 lav Exp $ */
-
 #include <config.h>
 
 extern "C" {
@@ -41,10 +38,9 @@ extern "C" {
 # include <termcap.h>
 #endif
 }
-
+#include <stdlib.h>
 #include "lftp_tinfo.h"
 
-#if defined(HAVE_TIGETSTR)
 static bool terminfo_ok = true;
 static void init_terminfo()
 {
@@ -52,21 +48,27 @@ static void init_terminfo()
    if(initted) return;
    initted = true;
 
+#if defined(HAVE_TIGETSTR)
    int errret=0;
    if(setupterm(NULL, 1, &errret) == ERR)
       terminfo_ok = false;
-}
+#elif defined(HAVE_TGETSTR)
+   static char buf[2048];
+   if(tgetent(buf,getenv("TERM")) == -1)
+      terminfo_ok = false;
 #endif
+}
 
 const char *get_string_term_cap(const char *terminfo_cap, const char *tcap_cap)
 {
-#if defined(HAVE_TIGETSTR)
    init_terminfo();
-   if(terminfo_ok) {
-      /* Cast to work around missing const def in some ncurses installations: */
-      const char *ret = tigetstr(const_cast<char *>(terminfo_cap));
-      if(ret && ret != (char *)-1) return ret;
-   }
+   if(!terminfo_ok)
+      return 0;
+
+#if defined(HAVE_TIGETSTR)
+   /* Cast to work around missing const def in some ncurses installations: */
+   const char *ret = tigetstr(const_cast<char *>(terminfo_cap));
+   if(ret && ret != (char *)-1) return ret;
 #elif defined(HAVE_TGETSTR)
    const char *ret = tgetstr(const_cast<char *>(tcap_cap), 0);
    if(ret && ret != (const char *)-1)
